//#ifndef __CINT__
#include "RooGlobalFunc.h"
//#endif
#include "RooRealVar.h"
#include "RooGenericPdf.h"
#include "RooArgList.h"
#include "RooFormulaVar.h"
#include "RooDataSet.h"
#include "RooGaussian.h"
#include "RooChebychev.h"
#include "RooExponential.h"
#include "RooBifurGauss.h"
#include "RooAddModel.h"
#include "RooProdPdf.h"
#include "TCanvas.h"
#include "RooPlot.h"
#include "RooHist.h"
#include "RooCBShape.h"
#include "RooPolynomial.h"
#include "RooBinning.h"
#include "TH1.h"
#include "TH2.h"
#include "RooAddPdf.h"
#include "RooProdPdf.h"
#include "RooFitResult.h"
#include "RooGenericPdf.h"
#include "RooLandau.h"
#include "TChain.h"
#include "TCanvas.h"
#include "TAxis.h"
#include "RooPlot.h"
#include "RooCategory.h"
#include "RooSuperCategory.h"
#include "RooSimultaneous.h"
#include "RooNLLVar.h"
#include "TFile.h"
#include "RooFit.h"
#include "RooGaussModel.h"
#include "RooHistPdf.h"
#include "RooPolyVar.h"

#include "myRooGaussian.cpp"

using namespace RooFit ;
using namespace std;



void unbinned_Kpipi0_RS_2D_fit()  
{
	
  // get the data in a TTree format
  TFile *f = new TFile("DtoKpipi0_RS_mc14ri_a_2Dfit.root");
  TTree *tree = (TTree*)f->Get("DstD0PiKPiPi0RS_withcut");
  
  // check we have the tree
  cout << "Entries = " << tree->GetEntries() << endl;

  //create both fit observables and get the data in RooFit land
  RooRealVar m("pi0_m_prefit","#it{m}(#it{#pi}^{0})", 0.1, 0.18, "GeV/#it{c}^{2}");
  RooRealVar errm("pi0_errm","#it{m}(#it{#pi}^{0}) error", 0, 0.015, "GeV/#it{c}^{2}");
  RooDataSet* dataxy = new RooDataSet("data","data",tree,RooArgSet(m, errm));

  
  //*  S i n g l e    G a u s s i o n     R e s o l u t i o n    f u n c t i o n
  //  ------------------------------------------------------------------------

  //Create function f(y) = a0 + a1*y (consider the variation of the mean as a function of mass error)
  RooRealVar p0("p0", "p0", 0.131985, 0, 10); //0.131985  
  RooRealVar p1("p1", "p1", 0.346866, 0, 10); //0.346866
  RooPolyVar mu("mu", "mean", errm, RooArgSet(p0, p1));

  //Create function f(y) = 1 + a/x + b/x^2 
  RooRealVar a("a", "a", -5.71988e-03, -1e-01, 1e-01); //-2.71988e-03  
  RooRealVar b("b", "b", 4.42226e-05, -1e-03, 1e-03);  //1.42226e-05
 
  RooFormulaVar sigma("sigma", "@0+@1+@2/@0", RooArgList(errm,a,b));

  myRooGaussian gauss("gauss", "1st gaussian PDF", m, mu, sigma);
  //RooGaussian gauss("gauss", "1st gaussian PDF", m, mu, sigma);


  // C o n s t r u c t  p d f   f o r   e r r o r   m a s s
  // -----------------------------------------------------------------
  
  // Construct a histogram pdf to describe the shape of the ErrM distribution
  RooDataSet datamerr("datamerr","datamerr", tree, RooArgSet(errm));

  RooDataHist *expHistDterr = datamerr.binnedClone();
  RooHistPdf pdfErr("pdfErr", "pdfErr", errm, *expHistDterr);
 

  // C o n s t r u c t   c o n d i t i o n a l   p r o d u c t   d e c a y 
   // -----------------------------------------------------------------------------
  
  // Construct production of conditional decay_dm(dt|dterr) with empirical pdfErr(dterr)
  RooProdPdf pdf("pdf", "pdf(m|sigma_m)*pdf(sigma_m)", pdfErr, RooFit::Conditional(gauss, m));

  m.setRange("SB1",0.1,0.18) ;
  errm.setRange("SB2",0,0.0031) ;
  
  //perform the fit
  pdf.fitTo(*dataxy, RooFit::Save(true), RooFit::Strategy(2));
  
 /*-----------------------------------PLOT THE RESULTS-------------------------------------*/
  
  //Plot pi0_M_prefit projections
  RooPlot *xframe = m.frame();
  dataxy->plotOn(xframe, CutRange("SB2"));
  const double nData = dataxy->sumEntries("", "SB2");
  pdf.plotOn(xframe, ProjWData(*dataxy));
  //pdf.plotOn(xframe);
  pdf.paramOn(xframe);

  //Plot pi0_ErrM projections
  RooPlot *yframe = errm.frame();
  dataxy->plotOn(yframe);
  pdf.plotOn(yframe);
  
  // pull distribution of pi0_M_prefit
  RooHist* hpull = xframe->pullHist();
  hpull->SetFillStyle(1001);
  hpull->SetFillColor(1);
  for(int i=0;i<hpull->GetN();++i) hpull->SetPointError(i,0.0,0.0,0.0,0.0);
  RooPlot* pullplot = m.frame(Title(" "));
  pullplot->addPlotable(hpull,"B");
  pullplot->SetYTitle("Pull mass");
  pullplot->SetMinimum(-4.);
  pullplot->SetMaximum(4.);
  pullplot->GetXaxis()->SetLabelSize(0.1);
  pullplot->GetYaxis()->SetLabelSize(0.07);
  
  
  TCanvas *canvas = new TCanvas("canvas","canvas", 700, 800);  //  700, 800
 
  Double_t xlow, ylow, xup, yup;
  canvas->GetPad(0)->GetPadPar(xlow,ylow,xup,yup);
  canvas->Divide(1,2);

  TVirtualPad *upPad = canvas->GetPad(1);
  upPad->SetPad(xlow,ylow+0.25*(yup-ylow),xup,yup);
  
  TVirtualPad *dwPad = canvas->GetPad(2);
  dwPad->SetPad(xlow,ylow,xup,ylow+0.25*(yup-ylow));

  canvas->Update();
  canvas->cd(1);
  pdf.plotOn(xframe,Components(gauss),LineStyle(kDashed),LineColor(kRed), ProjWData(*dataxy), Normalization(nData, RooAbsReal::NumEvent), ProjectionRange("SB2"));
  xframe->Draw();
	
  
  canvas->cd(2);
  pullplot->Draw();


}


  